@@ -1,51 +1,53 @@
-/** \file
- * Code for handling extensions (i.e.\ scripts).
- */
-/*
+/**
+ * Code for handling extensions (i.e. scripts).
+ *
  * Authors:
  *   Bryce Harrington <bryce@osdl.org>
  *   Ted Gould <ted@gould.cx>
  *   Jon A. Cruz <jon@joncruz.org>
+ *   Abhishek Sharma
  *
- * Copyright (C) 2002-2005,2007 Authors
+ * Copyright (C) 2002-2007 Authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#define __INKSCAPE_EXTENSION_IMPLEMENTATION_SCRIPT_C__
-
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
-
+#include <glibmm.h>
+#include <gtkmm/messagedialog.h>
+#include <gtkmm/main.h>
+#include <gtkmm/scrolledwindow.h>
+#include <gtkmm/textview.h>
+#include <glibmm/miscutils.h>
+#include <glibmm/convert.h>
 #include <unistd.h>
 
 #include <errno.h>
-#include <glib.h>
 #include <glib/gstdio.h>
-#include <gtkmm.h>
 
-#include "ui/view/view.h"
-#include "desktop-handles.h"
+
 #include "desktop.h"
-#include "selection.h"
-#include "sp-namedview.h"
-#include "io/sys.h"
-#include "preferences.h"
-#include "../system.h"
+#include "ui/dialog-events.h"
 #include "extension/effect.h"
 #include "extension/output.h"
 #include "extension/input.h"
 #include "extension/db.h"
+#include "inkscape.h"
+#include "io/sys.h"
+#include "preferences.h"
 #include "script.h"
-#include "dialogs/dialog-events.h"
-#include "application/application.h"
+#include "selection.h"
+#include "sp-namedview.h"
+#include "extension/system.h"
+#include "ui/view/view.h"
 #include "xml/node.h"
 #include "xml/attribute-record.h"
-
-#include "util/glib-list-iterators.h"
+#include "ui/tools/node-tool.h"
+#include "ui/tool/multi-path-manipulator.h"
+#include "ui/tool/path-manipulator.h"
+#include "ui/tool/control-point-selection.h"
 
 
+#include "path-prefix.h"
 
 #ifdef WIN32
 #include <windows.h>
@@ -53,13 +55,9 @@
 #include "registrytool.h"
 #endif
 
-
-
 /** This is the command buffer that gets allocated from the stack */
 #define BUFSIZE (255)
 
-
-
 /* Namespaces */
 namespace Inkscape {
 namespace Extension {
@@ -148,12 +146,13 @@ std::string Script::resolveInterpreterExecutable(const Glib::ustring &interpName
    of memory in the unloaded state.
 */
 Script::Script() :
-    Implementation()
+    Implementation(),
+    _canceled(false)
 {
 }
 
 /**
- *   brief     Destructor
+ *   \brief     Destructor
  */
 Script::~Script()
 {
@@ -176,30 +175,28 @@ Script::~Script()
     string.  This means that the caller of this function can always
     free what they are given (and should do it too!).
 */
-std::string
-Script::solve_reldir(Inkscape::XML::Node *reprin) {
+std::string Script::solve_reldir(Inkscape::XML::Node *reprin) {
 
     gchar const *s = reprin->attribute("reldir");
 
     // right now the only recognized relative directory is "extensions"
     if (!s || Glib::ustring(s) != "extensions") {
-        Glib::ustring str = sp_repr_children(reprin)->content();
+        Glib::ustring str = reprin->firstChild()->content();
         return str;
     }
 
     Glib::ustring reldir = s;
-
     for (unsigned int i=0;
         i < Inkscape::Extension::Extension::search_path.size();
         i++) {
 
         gchar * fname = g_build_filename(
            Inkscape::Extension::Extension::search_path[i],
-           sp_repr_children(reprin)->content(),
+           reprin->firstChild()->content(),
            NULL);
         Glib::ustring filename = fname;
         g_free(fname);
-
+        //printf("Filename: %s\n", filename.c_str());
         if ( Inkscape::IO::file_test(filename.c_str(), G_FILE_TEST_EXISTS) ) {
             return Glib::filename_from_utf8(filename);
         }
@@ -228,27 +225,21 @@ Script::solve_reldir(Inkscape::XML::Node *reprin) {
 */
 bool Script::check_existence(const std::string &command)
 {
-
     // Check the simple case first
     if (command.empty()) {
         return false;
     }
 
     //Don't search when it is an absolute path. */
-    if (!Glib::path_is_absolute(command)) {
-        if (Glib::file_test(command, Glib::FILE_TEST_EXISTS)) {
-            return true;
-        } else {
-            return false;
-        }
+    if (Glib::path_is_absolute(command)) {
+        return Glib::file_test(command, Glib::FILE_TEST_EXISTS);
     }
 
-    std::string path = Glib::getenv("PATH");
-    if (path.empty()) {
-       /* There is no `PATH' in the environment.
-           The default search path is the current directory */
-        path = G_SEARCHPATH_SEPARATOR_S;
-    }
+    // First search in the current directory
+    std::string path = G_SEARCHPATH_SEPARATOR_S;
+    path.append(";");
+    // And then in the PATH environment variable.
+    path.append(Glib::getenv("PATH"));
 
     std::string::size_type pos  = 0;
     std::string::size_type pos2 = 0;
@@ -285,9 +276,9 @@ bool Script::check_existence(const std::string &command)
 
 /**
     \return   none
-    \brief    This function 'loads' an extention, basically it determines
-              the full command for the extention and stores that.
-    \param    module  The extention to be loaded.
+    \brief    This function 'loads' an extension, basically it determines
+              the full command for the extension and stores that.
+    \param    module  The extension to be loaded.
 
     The most difficult part about this function is finding the actual
     command through all of the Reprs.  Basically it is hidden down a
@@ -297,7 +288,7 @@ bool Script::check_existence(const std::string &command)
 
     At that point all of the loops are exited, and there is an
     if statement to make sure they didn't exit because of not finding
-    the command.  If that's the case, the extention doesn't get loaded
+    the command.  If that's the case, the extension doesn't get loaded
     and should error out at a higher level.
 */
 
@@ -310,28 +301,28 @@ bool Script::load(Inkscape::Extension::Extension *module)
     helper_extension = "";
 
     /* This should probably check to find the executable... */
-    Inkscape::XML::Node *child_repr = sp_repr_children(module->get_repr());
+    Inkscape::XML::Node *child_repr = module->get_repr()->firstChild();
     while (child_repr != NULL) {
         if (!strcmp(child_repr->name(), INKSCAPE_EXTENSION_NS "script")) {
-            child_repr = sp_repr_children(child_repr);
+            child_repr = child_repr->firstChild();
             while (child_repr != NULL) {
                 if (!strcmp(child_repr->name(), INKSCAPE_EXTENSION_NS "command")) {
                     const gchar *interpretstr = child_repr->attribute("interpreter");
                     if (interpretstr != NULL) {
                         std::string interpString = resolveInterpreterExecutable(interpretstr);
-                        command.insert(command.end(), interpString);
+                        command.push_back(interpString);
                     }
-                    command.insert(command.end(), solve_reldir(child_repr));
+                    command.push_back(solve_reldir(child_repr));
                 }
                 if (!strcmp(child_repr->name(), INKSCAPE_EXTENSION_NS "helper_extension")) {
-                    helper_extension = sp_repr_children(child_repr)->content();
+                    helper_extension = child_repr->firstChild()->content();
                 }
-                child_repr = sp_repr_next(child_repr);
+                child_repr = child_repr->next();
             }
 
             break;
         }
-        child_repr = sp_repr_next(child_repr);
+        child_repr = child_repr->next();
     }
 
     //g_return_val_if_fail(command.length() > 0, false);
@@ -363,39 +354,41 @@ void Script::unload(Inkscape::Extension::Extension */*module*/)
     \param    module  The Extension in question
 
 */
-bool
-Script::check(Inkscape::Extension::Extension *module)
+bool Script::check(Inkscape::Extension::Extension *module)
 {
     int script_count = 0;
-    Inkscape::XML::Node *child_repr = sp_repr_children(module->get_repr());
+    Inkscape::XML::Node *child_repr = module->get_repr()->firstChild();
     while (child_repr != NULL) {
         if (!strcmp(child_repr->name(), INKSCAPE_EXTENSION_NS "script")) {
             script_count++;
-            child_repr = sp_repr_children(child_repr);
+            child_repr = child_repr->firstChild();
             while (child_repr != NULL) {
                 if (!strcmp(child_repr->name(), INKSCAPE_EXTENSION_NS "check")) {
                     std::string command_text = solve_reldir(child_repr);
                     if (!command_text.empty()) {
                         /* I've got the command */
                         bool existance = check_existence(command_text);
-                        if (!existance)
+                        if (!existance) {
                             return false;
+                        }
+                    } else {
+                        return false;
                     }
                 }
 
                 if (!strcmp(child_repr->name(), INKSCAPE_EXTENSION_NS "helper_extension")) {
-                    gchar const *helper = sp_repr_children(child_repr)->content();
+                    gchar const *helper = child_repr->firstChild()->content();
                     if (Inkscape::Extension::db.get(helper) == NULL) {
                         return false;
                     }
                 }
 
-                child_repr = sp_repr_next(child_repr);
+                child_repr = child_repr->next();
             }
 
             break;
         }
-        child_repr = sp_repr_next(child_repr);
+        child_repr = child_repr->next();
     }
 
     if (script_count == 0) {
@@ -532,9 +525,8 @@ SPDocument *Script::open(Inkscape::Extension::Input *module,
     } // data_read
 
     if (mydoc != NULL) {
-        g_free(mydoc->base);
-        mydoc->base = NULL;
-        sp_document_change_uri_and_hrefs(mydoc, filenameArg);
+        mydoc->setBase(0);
+        mydoc->changeUriAndHrefs(filenameArg);
     }
 
     // make sure we don't leak file descriptors from g_file_open_tmp
@@ -549,17 +541,17 @@ SPDocument *Script::open(Inkscape::Extension::Input *module,
 
 /**
     \return   none
-    \brief    This function uses an extention to save a document.  It first
+    \brief    This function uses an extension to save a document.  It first
               creates an SVG file of the document, and then runs it through
               the script.
-    \param    module    Extention to be used
+    \param    module    Extension to be used
     \param    doc       Document to be saved
     \param    filename  The name to save the final file as
     \return   false in case of any failure writing the file, otherwise true
 
     Well, at some point people need to save - it is really what makes
     the entire application useful.  And, it is possible that someone
-    would want to use an extetion for this, so we need a function to
+    would want to use an extension for this, so we need a function to
     do that eh?
 
     First things first, the document is saved to a temporary file that
@@ -567,7 +559,7 @@ SPDocument *Script::open(Inkscape::Extension::Input *module,
     ink_ext_ as a prefix.  Don't worry, this file gets deleted at the
     end of the function.
 
-    After we have the SVG file, then extention_execute is called with
+    After we have the SVG file, then Script::execute is called with
     the temporary file name and the final output filename.  This should
     put the output of the script into the final output file.  We then
     delete the temporary file.
@@ -602,10 +594,14 @@ void Script::save(Inkscape::Extension::Output *module,
 
 
     file_listener fileout;
-    execute(command, params, tempfilename_in, fileout);
+    int data_read = execute(command, params, tempfilename_in, fileout);
 
-    std::string lfilename = Glib::filename_from_utf8(filenameArg);
-    bool success = fileout.toFile(lfilename);
+    bool success = false;
+
+    if (data_read > 0) {
+        std::string lfilename = Glib::filename_from_utf8(filenameArg);
+        success = fileout.toFile(lfilename);
+    }
 
     // make sure we don't leak file descriptors from g_file_open_tmp
     close(tempfd_in);
@@ -661,8 +657,13 @@ void Script::effect(Inkscape::Extension::Effect *module,
         printf("TOO BAD TO LIVE!!!");
         exit(1);
     }
+    if (doc == NULL)
+    {
+        g_warning("Script::effect: View not defined");
+        return;
+    }
 
-    SPDesktop *desktop = (SPDesktop *)doc;
+    SPDesktop *desktop = reinterpret_cast<SPDesktop *>(doc);
     sp_namedview_document_from_window(desktop);
 
     std::list<std::string> params;
@@ -688,17 +689,57 @@ void Script::effect(Inkscape::Extension::Effect *module,
         return;
     }
 
-    if (desktop != NULL) {
-        Inkscape::Util::GSListConstIterator<SPItem *> selected =
-             sp_desktop_selection(desktop)->itemList();
-        while ( selected != NULL ) {
-            Glib::ustring selected_id;
-            selected_id += "--id=";
-            selected_id += (*selected)->getId();
-            params.insert(params.begin(), selected_id);
-            ++selected;
+    std::vector<SPItem*> selected =
+        desktop->getSelection()->itemList(); //desktop should not be NULL since doc was checked and desktop is a casted pointer
+    for(std::vector<SPItem*>::const_iterator x = selected.begin(); x != selected.end(); ++x){
+        Glib::ustring selected_id;
+        selected_id += "--id=";
+        selected_id += (*x)->getId();
+        params.push_front(selected_id);
+    }
+
+    {//add selected nodes
+    Inkscape::UI::Tools::NodeTool *tool = 0;
+    if (SP_ACTIVE_DESKTOP ) {
+        Inkscape::UI::Tools::ToolBase *ec = SP_ACTIVE_DESKTOP->event_context;
+        if (INK_IS_NODE_TOOL(ec)) {
+            tool = static_cast<Inkscape::UI::Tools::NodeTool*>(ec);
         }
     }
+    
+    if(tool){
+        Inkscape::UI::ControlPointSelection *cps = tool->_selected_nodes;
+        for (Inkscape::UI::ControlPointSelection::iterator i = cps->begin(); i != cps->end(); ++i) {
+            Inkscape::UI::Node *node = dynamic_cast<Inkscape::UI::Node*>(*i);
+            if (node) { 
+                std::string id = node->nodeList().subpathList().pm().item()->getId(); 
+
+                int sp = 0;
+                bool found_sp = false;
+                for(Inkscape::UI::SubpathList::iterator i = node->nodeList().subpathList().begin(); i != node->nodeList().subpathList().end(); ++i,++sp){
+                    if(&**i == &(node->nodeList())){
+                        found_sp = true;
+                        break;
+                    }
+                }
+                int nl=0;
+                bool found_nl = false;
+                for (Inkscape::UI::NodeList::iterator j = node->nodeList().begin(); j != node->nodeList().end(); ++j, ++nl){
+                    if(&*j==node){
+                        found_nl = true;
+                        break;
+                    }
+                }
+                std::ostringstream ss;
+                ss<< "--selected-nodes=" << id << ":" << sp << ":" << nl;
+                Glib::ustring selected = ss.str();
+
+                if(found_nl && found_sp)params.push_front(selected);
+                else g_warning("Something went wrong while trying to pass selected nodes to extension. Please report a bug.");
+            }
+        }
+    }
+    }//end add selected nodes
 
     file_listener fileout;
     int data_read = execute(command, params, dc->_filename, fileout);
@@ -720,13 +761,39 @@ void Script::effect(Inkscape::Extension::Effect *module,
 
     g_unlink(tempfilename_out.c_str());
 
-    /* Do something with mydoc.... */
     if (mydoc) {
-        doc->doc()->emitReconstructionStart();
-        copy_doc(doc->doc()->rroot, mydoc->rroot);
-        doc->doc()->emitReconstructionFinish();
+        SPDocument* vd=doc->doc();
+        if (vd != NULL)
+        {
+            vd->emitReconstructionStart();
+            copy_doc(vd->rroot, mydoc->rroot);
+            vd->emitReconstructionFinish();
+
+            // Getting the named view from the document generated by the extension
+            SPNamedView *nv = sp_document_namedview(mydoc, NULL);
+            
+            //Check if it has a default layer set up
+            SPObject *layer = NULL;
+            if ( nv != NULL)
+            {
+                if( nv->default_layer_id != 0 ) {
+                    SPDocument *document = desktop->doc();
+                    //If so, get that layer
+                    if (document != NULL)
+                    {
+                        layer = document->getObjectById(g_quark_to_string(nv->default_layer_id));
+                    }
+                }
+            }
+            
+            sp_namedview_update_layers_from_document(desktop);
+            //If that layer exists,
+            if (layer) {
+                //set the current layer
+                desktop->setCurrentLayer(layer);
+            }
+        }
         mydoc->release();
-        sp_namedview_update_layers_from_document(desktop);
     }
 
     return;
@@ -735,25 +802,68 @@ void Script::effect(Inkscape::Extension::Effect *module,
 
 
 /**
-    \brief  A function to take all the svg elements from one document
-            and put them in another.
-    \param  oldroot  The root node of the document to be replaced
-    \param  newroot  The root node of the document to replace it with
+    \brief  A function to replace all the elements in an old document
+            by those from a new document.
+            document and repinserts them into an emptied old document.
+    \param  oldroot  The root node of the old (destination) document.
+    \param  newroot  The root node of the new (source) document.
 
-    This function first deletes all of the data in the old document.  It
-    does this by creating a list of what needs to be deleted, and then
-    goes through the list.  This two pass approach removes issues with
-    the list being change while parsing through it.  Lots of nasty bugs.
+    This function first deletes all the root attributes in the old document followed
+    by copying all the root attributes from the new document to the old document.
 
-    Then, it goes through the new document, duplicating all of the
-    elements and putting them into the old document.  The copy
-    is then complete.
+    It then deletes all the elements in the old document by
+    making two pass, the first to create a list of the old elements and
+    the second to actually delete them. This two pass approach removes issues
+    with the list being change while parsing through it... lots of nasty bugs.
+
+    Then, it copies all the element in the new document into the old document.
+
+    Finally, it copies the attributes in namedview.
 */
 void Script::copy_doc (Inkscape::XML::Node * oldroot, Inkscape::XML::Node * newroot)
 {
+    if ((oldroot == NULL) ||(newroot == NULL))
+    {
+        g_warning("Error on copy_doc: NULL pointer input.");
+        return;
+    }
+
+    // For copying attributes in root and in namedview
+    using Inkscape::Util::List;
+    using Inkscape::XML::AttributeRecord;
+    std::vector<gchar const *> attribs;
+
+    // Must explicitly copy root attributes. This must be done first since
+    // copying grid lines calls "SPGuide::set()" which needs to know the
+    // width, height, and viewBox of the root element.
+
+    // Make a list of all attributes of the old root node.
+    for (List<AttributeRecord const> iter = oldroot->attributeList(); iter; ++iter) {
+        attribs.push_back(g_quark_to_string(iter->key));
+    }
+
+    // Delete the attributes of the old root node.
+    for (std::vector<gchar const *>::const_iterator it = attribs.begin(); it != attribs.end(); ++it) {
+        oldroot->setAttribute(*it, NULL);
+    }
+
+    // Set the new attributes.
+    for (List<AttributeRecord const> iter = newroot->attributeList(); iter; ++iter) {
+        gchar const *name = g_quark_to_string(iter->key);
+        oldroot->setAttribute(name, newroot->attribute(name));
+    }
+
+
+    // Question: Why is the "sodipodi:namedview" special? Treating it as a normal
+    // elmement results in crashes.
+    // Seems to be a bug:
+    // http://inkscape.13.x6.nabble.com/Effect-that-modifies-the-document-properties-tt2822126.html
+
     std::vector<Inkscape::XML::Node *> delete_list;
     Inkscape::XML::Node * oldroot_namedview = NULL;
+    Inkscape::XML::Node * newroot_namedview = NULL;
 
+    // Make list
     for (Inkscape::XML::Node * child = oldroot->firstChild();
             child != NULL;
             child = child->next()) {
@@ -768,46 +878,51 @@ void Script::copy_doc (Inkscape::XML::Node * oldroot, Inkscape::XML::Node * newr
             delete_list.push_back(child);
         }
     }
+
+    if(!oldroot_namedview)
+    {
+        g_warning("Error on copy_doc: No namedview on destination document.");
+        return;
+    }
+
+    // Unparent (delete)
     for (unsigned int i = 0; i < delete_list.size(); i++) {
         sp_repr_unparent(delete_list[i]);
     }
 
+    // Copy
     for (Inkscape::XML::Node * child = newroot->firstChild();
             child != NULL;
             child = child->next()) {
         if (!strcmp("sodipodi:namedview", child->name())) {
-            if (oldroot_namedview != NULL) {
-                for (Inkscape::XML::Node * newroot_namedview_child = child->firstChild();
-                        newroot_namedview_child != NULL;
-                        newroot_namedview_child = newroot_namedview_child->next()) {
-                    oldroot_namedview->appendChild(newroot_namedview_child->duplicate(oldroot->document()));
-                }
+            newroot_namedview = child;
+            for (Inkscape::XML::Node * newroot_namedview_child = child->firstChild();
+                    newroot_namedview_child != NULL;
+                    newroot_namedview_child = newroot_namedview_child->next()) {
+                oldroot_namedview->appendChild(newroot_namedview_child->duplicate(oldroot->document()));
             }
         } else {
             oldroot->appendChild(child->duplicate(oldroot->document()));
         }
     }
 
-    {
-        using Inkscape::Util::List;
-        using Inkscape::XML::AttributeRecord;
-        std::vector<gchar const *> attribs;
+    attribs.clear();
 
-        // Make a list of all attributes of the old root node.
-        for (List<AttributeRecord const> iter = oldroot->attributeList(); iter; ++iter) {
-            attribs.push_back(g_quark_to_string(iter->key));
-        }
+    // Must explicitly copy namedview attributes.
+    // Make a list of all attributes of the old namedview node.
+    for (List<AttributeRecord const> iter = oldroot_namedview->attributeList(); iter; ++iter) {
+        attribs.push_back(g_quark_to_string(iter->key));
+    }
 
-        // Delete the attributes of the old root nodes.
-        for (std::vector<gchar const *>::const_iterator it = attribs.begin(); it != attribs.end(); it++) {
-            oldroot->setAttribute(*it, NULL);
-        }
+    // Delete the attributes of the old namedview node.
+    for (std::vector<gchar const *>::const_iterator it = attribs.begin(); it != attribs.end(); ++it) {
+        oldroot_namedview->setAttribute(*it, NULL);
+    }
 
-        // Set the new attributes.
-        for (List<AttributeRecord const> iter = newroot->attributeList(); iter; ++iter) {
-            gchar const *name = g_quark_to_string(iter->key);
-            oldroot->setAttribute(name, newroot->attribute(name));
-        }
+    // Set the new attributes.
+    for (List<AttributeRecord const> iter = newroot_namedview->attributeList(); iter; ++iter) {
+        gchar const *name = g_quark_to_string(iter->key);
+        oldroot_namedview->setAttribute(name, newroot_namedview->attribute(name));
     }
 
     /** \todo  Restore correct layer */
@@ -827,7 +942,11 @@ void Script::checkStderr (const Glib::ustring &data,
     GtkWidget *dlg = GTK_WIDGET(warning.gobj());
     sp_transientize(dlg);
 
-    Gtk::VBox * vbox = warning.get_vbox();
+#if WITH_GTKMM_3_0
+    Gtk::Box * vbox = warning.get_content_area();
+#else
+    Gtk::Box * vbox = warning.get_vbox();
+#endif
 
     /* Gtk::TextView * textview = new Gtk::TextView(Gtk::TextBuffer::create()); */
     Gtk::TextView * textview = new Gtk::TextView();
@@ -835,19 +954,14 @@ void Script::checkStderr (const Glib::ustring &data,
     textview->set_wrap_mode(Gtk::WRAP_WORD);
     textview->show();
 
-    // Remove the last character
-    char *errormsg = (char*) data.c_str();
-    while (*errormsg != '\0') errormsg++;
-    errormsg -= 1;
-    *errormsg = '\0';
-
-    textview->get_buffer()->set_text(_(data.c_str()));
+    textview->get_buffer()->set_text(data.c_str());
 
     Gtk::ScrolledWindow * scrollwindow = new Gtk::ScrolledWindow();
     scrollwindow->add(*textview);
     scrollwindow->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
     scrollwindow->set_shadow_type(Gtk::SHADOW_IN);
     scrollwindow->show();
+    scrollwindow->set_size_request(0, 60);
 
     vbox->pack_start(*scrollwindow, true, true, 5 /* fix these */);
 
@@ -936,9 +1050,18 @@ int Script::execute (const std::list<std::string> &in_command,
     // assemble the rest of argv
     std::copy(in_params.begin(), in_params.end(), std::back_inserter(argv));
     if (!filein.empty()) {
-        argv.push_back(filein);
+        if(Glib::path_is_absolute(filein))
+            argv.push_back(filein);
+        else {
+            std::vector<std::string> buildargs;
+            buildargs.push_back(Glib::get_current_dir());
+            buildargs.push_back(filein);
+            argv.push_back(Glib::build_filename(buildargs));
+        }
     }
 
+    //for(int i=0;i<argv.size(); ++i){printf("%s ",argv[i].c_str());}printf("\n");
+
     int stdout_pipe, stderr_pipe;
 
     try {
@@ -950,12 +1073,15 @@ int Script::execute (const std::list<std::string> &in_command,
                                      NULL,           // STDIN
                                      &stdout_pipe,   // STDOUT
                                      &stderr_pipe);  // STDERR
-    } catch (Glib::Error e) {
+    } catch (Glib::Error &e) {
         printf("Can't Spawn!!! spawn returns: %s\n", e.what().data());
         return 0;
     }
 
-    _main_loop = Glib::MainLoop::create(false);
+    // Create a new MainContext for the loop so that the original context sources are not run here,
+    // this enforces that only the file_listeners should be read in this new MainLoop
+    Glib::RefPtr<Glib::MainContext> _main_context = Glib::MainContext::create();
+    _main_loop = Glib::MainLoop::create(_main_context, false);
 
     file_listener fileerr;
     fileout.init(stdout_pipe, _main_loop);
@@ -979,7 +1105,7 @@ int Script::execute (const std::list<std::string> &in_command,
 
     Glib::ustring stderr_data = fileerr.string();
     if (stderr_data.length() != 0 &&
-        Inkscape::NSApplication::Application::getUseGui()
+        INKSCAPE.use_gui()
        ) {
         checkStderr(stderr_data, Gtk::MESSAGE_INFO,
                                  _("Inkscape has received additional data from the script executed.  "
@@ -996,7 +1122,45 @@ int Script::execute (const std::list<std::string> &in_command,
 }
 
 
+void Script::file_listener::init(int fd, Glib::RefPtr<Glib::MainLoop> main) {
+    _channel = Glib::IOChannel::create_from_fd(fd);
+    _channel->set_encoding();
+    _conn = main->get_context()->signal_io().connect(sigc::mem_fun(*this, &file_listener::read), _channel, Glib::IO_IN | Glib::IO_HUP | Glib::IO_ERR);
+    _main_loop = main;
 
+    return;
+}
+
+bool Script::file_listener::read(Glib::IOCondition condition) {
+    if (condition != Glib::IO_IN) {
+        _main_loop->quit();
+        return false;
+    }
+
+    Glib::IOStatus status;
+    Glib::ustring out;
+    status = _channel->read_line(out);
+    _string += out;
+
+    if (status != Glib::IO_STATUS_NORMAL) {
+        _main_loop->quit();
+        _dead = true;
+        return false;
+    }
+
+    return true;
+}
+
+bool Script::file_listener::toFile(const Glib::ustring &name) {
+    try {
+        Glib::RefPtr<Glib::IOChannel> stdout_file = Glib::IOChannel::create_from_file(name, "w");
+        stdout_file->set_encoding();
+        stdout_file->write(_string);
+    } catch (Glib::FileError &e) {
+        return false;
+    }
+    return true;
+}
 
 }  // namespace Implementation
 }  // namespace Extension
@@ -1011,4 +1175,4 @@ int Script::execute (const std::list<std::string> &in_command,
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8 :
