@@ -6,7 +6,7 @@
  */
 /*
  * Authors:
- * 	   Miklos Erdelyi <erdelyim@gmail.com>
+ *     Miklos Erdelyi <erdelyim@gmail.com>
  *
  * Copyright (C) 2006 Miklos Erdelyi
  *
@@ -21,8 +21,8 @@
 #include <set>
 #include <string>
 
-#include "libnr/nr-path.h"
-#include <libnr/nr-matrix-ops.h>
+#include <2geom/forward.h>
+#include <2geom/affine.h>
 
 #include "style.h"
 
@@ -31,14 +31,19 @@
 class SPClipPath;
 class SPMask;
 
+typedef struct _PangoFont PangoFont;
+typedef struct _PangoLayout PangoLayout;
+
 namespace Inkscape {
+class Pixbuf;
+
 namespace Extension {
 namespace Internal {
 
 class CairoRenderer;
 class CairoRenderContext;
-class CairoRenderState;
-class CairoGlyphInfo;
+struct CairoRenderState;
+struct CairoGlyphInfo;
 
 // Holds info for rendering a glyph
 struct CairoGlyphInfo {
@@ -49,16 +54,17 @@ struct CairoGlyphInfo {
 
 struct CairoRenderState {
     unsigned int merge_opacity : 1;     // whether fill/stroke opacity can be mul'd with item opacity
-    unsigned int need_layer : 1;
+    unsigned int need_layer : 1;        // whether object is masked, clipped, and/or has a non-zero opacity
     unsigned int has_overflow : 1;
     unsigned int parent_has_userspace : 1;  // whether the parent's ctm should be applied
     float opacity;
     bool has_filtereffect;
+    Geom::Affine item_transform;     // this item's item->transform, for correct clipping
 
     SPClipPath *clip_path;
     SPMask* mask;
 
-    NRMatrix transform;     // the CTM
+    Geom::Affine transform;     // the CTM
 };
 
 class CairoRenderContext {
@@ -71,12 +77,12 @@ public:
     CairoRenderer *getRenderer(void) const;
     cairo_t *getCairoContext(void) const;
 
-    typedef enum CairoRenderMode {
+    enum CairoRenderMode {
         RENDER_MODE_NORMAL,
         RENDER_MODE_CLIP
     };
 
-    typedef enum CairoClipMode {
+    enum CairoClipMode {
         CLIP_MODE_PATH,
         CLIP_MODE_MASK
     };
@@ -85,13 +91,16 @@ public:
     bool setPdfTarget(gchar const *utf8_fn);
     bool setPsTarget(gchar const *utf8_fn);
     /** Set the cairo_surface_t from an external source */
-    bool setSurfaceTarget(cairo_surface_t *surface, bool is_vector);
+    bool setSurfaceTarget(cairo_surface_t *surface, bool is_vector, cairo_matrix_t *ctm=NULL);
 
     void setPSLevel(unsigned int level);
+    void setEPS(bool eps);
     unsigned int getPSLevel(void);
     void setPDFLevel(unsigned int level);
     void setTextToPath(bool texttopath);
     bool getTextToPath(void);
+    void setOmitText(bool omittext);
+    bool getOmitText(void);
     void setFilterToBitmap(bool filtertobitmap);
     bool getFilterToBitmap(void);
     void setBitmapResolution(int resolution);
@@ -107,14 +116,17 @@ public:
     /** Saves the contents of the context to a PNG file. */
     bool saveAsPng(const char *file_name);
 
+    /** On targets supporting multiple pages, sends subsequent rendering to a new page*/
+    void newPage(void);
+
     /* Render/clip mode setting/query */
     void setRenderMode(CairoRenderMode mode);
     CairoRenderMode getRenderMode(void) const;
     void setClipMode(CairoClipMode mode);
     CairoClipMode getClipMode(void) const;
 
-    void addBpath(NArtBpath const *bp);
-    void setBpath(NArtBpath const *bp);
+    void addPathVector(Geom::PathVector const &pv);
+    void setPathVector(Geom::PathVector const &pv);
 
     void pushLayer(void);
     void popLayer(void);
@@ -126,20 +138,27 @@ public:
     CairoRenderState *getParentState(void) const;
     void setStateForStyle(SPStyle const *style);
 
-    void transform(NRMatrix const *transform);
-    void setTransform(NRMatrix const *transform);
-    void getTransform(NRMatrix *copy) const;
-    void getParentTransform(NRMatrix *copy) const;
+    void transform(Geom::Affine const &transform);
+    void setTransform(Geom::Affine const &transform);
+    Geom::Affine getTransform() const;
+    Geom::Affine getParentTransform() const;
 
     /* Clipping methods */
-    void addClipPath(NArtBpath const *bp, SPIEnum const *fill_rule);
+    void addClipPath(Geom::PathVector const &pv, SPIEnum const *fill_rule);
     void addClippingRect(double x, double y, double width, double height);
 
     /* Rendering methods */
-    bool renderPath(NRBPath const *bpath, SPStyle const *style, NRRect const *pbox);
-    bool renderImage(unsigned char *px, unsigned int w, unsigned int h, unsigned int rs,
-                     NRMatrix const *image_transform, SPStyle const *style);
-    bool renderGlyphtext(PangoFont *font, NRMatrix const *font_matrix,
+    enum CairoPaintOrder {
+        STROKE_OVER_FILL,
+        FILL_OVER_STROKE,
+        FILL_ONLY,
+        STROKE_ONLY
+    };
+
+    bool renderPathVector(Geom::PathVector const &pathv, SPStyle const *style, Geom::OptRect const &pbox, CairoPaintOrder order = STROKE_OVER_FILL);
+    bool renderImage(Inkscape::Pixbuf *pb,
+                     Geom::Affine const &image_transform, SPStyle const *style);
+    bool renderGlyphtext(PangoFont *font, Geom::Affine const &font_matrix,
                          std::vector<CairoGlyphInfo> const &glyphtext, SPStyle const *style);
 
     /* More general rendering methods will have to be added (like fill, stroke) */
@@ -148,12 +167,20 @@ protected:
     CairoRenderContext(CairoRenderer *renderer);
     virtual ~CairoRenderContext(void);
 
+    enum CairoOmitTextPageState {
+        EMPTY,
+        GRAPHIC_ON_TOP,
+        NEW_PAGE_ON_GRAPHIC
+    };
+
     float _width;
     float _height;
     unsigned short _dpi;
     unsigned int _pdf_level;
     unsigned int _ps_level;
+    bool _eps;
     bool _is_texttopath;
+    bool _is_omittext;
     bool _is_filtertobitmap;
     int _bitmapresolution;
 
@@ -162,7 +189,7 @@ protected:
     unsigned int _is_valid : 1;
     unsigned int _vector_based_target : 1;
 
-    cairo_t *_cr;
+    cairo_t *_cr; // Cairo context
     cairo_surface_t *_surface;
     cairo_surface_type_t _target;
     cairo_format_t _target_format;
@@ -179,19 +206,28 @@ protected:
     CairoRenderMode _render_mode;
     CairoClipMode _clip_mode;
 
+    CairoOmitTextPageState _omittext_state;
+
     cairo_pattern_t *_createPatternForPaintServer(SPPaintServer const *const paintserver,
-                                                  NRRect const *pbox, float alpha);
-    cairo_pattern_t *_createPatternPainter(SPPaintServer const *const paintserver, NRRect const *pbox);
+                                                  Geom::OptRect const &pbox, float alpha);
+    cairo_pattern_t *_createPatternPainter(SPPaintServer const *const paintserver, Geom::OptRect const &pbox);
+    cairo_pattern_t *_createHatchPainter(SPPaintServer const *const paintserver, Geom::OptRect const &pbox);
 
     unsigned int _showGlyphs(cairo_t *cr, PangoFont *font, std::vector<CairoGlyphInfo> const &glyphtext, bool is_stroke);
 
-    bool _finishSurfaceSetup(cairo_surface_t *surface);
-    void _setFillStyle(SPStyle const *style, NRRect const *pbox);
-    void _setStrokeStyle(SPStyle const *style, NRRect const *pbox);
+    bool _finishSurfaceSetup(cairo_surface_t *surface, cairo_matrix_t *ctm = NULL);
+    void _setFillStyle(SPStyle const *style, Geom::OptRect const &pbox);
+    void _setStrokeStyle(SPStyle const *style, Geom::OptRect const &pbox);
 
-    void _initCairoMatrix(cairo_matrix_t *matrix, NRMatrix const *transform);
+    void _initCairoMatrix(cairo_matrix_t *matrix, Geom::Affine const &transform);
     void _concatTransform(cairo_t *cr, double xx, double yx, double xy, double yy, double x0, double y0);
-    void _concatTransform(cairo_t *cr, NRMatrix const *transform);
+    void _concatTransform(cairo_t *cr, Geom::Affine const &transform);
+
+    void _prepareRenderGraphic(void);
+    void _prepareRenderText(void);
+
+    std::map<gpointer, cairo_font_face_t *> font_table;
+    static void font_data_free(gpointer data);
 
     CairoRenderState *_createState(void);
 };
@@ -211,4 +247,4 @@ protected:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
