@@ -177,6 +177,7 @@ enum {
     SP_ARG_SHELL,
     SP_ARG_VERSION,
     SP_ARG_VACUUM_DEFS,
+    SP_ARG_NO_CONVERT_TEXT_BASELINE_SPACING,
 #ifdef WITH_DBUS
     SP_ARG_DBUS_LISTEN,
     SP_ARG_DBUS_NAME,
@@ -241,7 +242,6 @@ static gchar *sp_export_png_utf8 = NULL;
 static gchar *sp_export_svg_utf8 = NULL;
 static gchar *sp_global_printer_utf8 = NULL;
 
-
 /**
  *  Reset variables to default values.
  */
@@ -280,6 +280,7 @@ static void resetCommandlineGlobals() {
         sp_query_all = FALSE;
         sp_query_id = NULL;
         sp_vacuum_defs = FALSE;
+        sp_no_convert_text_baseline_spacing = FALSE;
 #ifdef WITH_DBUS
         sp_dbus_listen = FALSE;
         sp_dbus_name = NULL;
@@ -421,7 +422,7 @@ struct poptOption options[] = {
     {"export-pdf-version", 0,
      POPT_ARG_STRING, &sp_export_pdf_version, SP_ARG_EXPORT_PDF_VERSION,
      // TRANSLATORS: "--export-pdf-version" is an Inkscape command line option; see "inkscape --help"
-     N_("Export PDF to given version. (hint: make sure to input the exact string found in the PDF export dialog, e.g. \"PDF 1.4\" which is PDF-a conformant)"),
+     N_("Export PDF to given version. (hint: make sure to input the exact string found in the PDF export dialog, e.g. \"1.4\" which is PDF-a conformant)"),
      N_("PDF_VERSION")},
 
     {"export-latex", 0,
@@ -526,6 +527,11 @@ struct poptOption options[] = {
      N_("Start Inkscape in interactive shell mode."),
      NULL},
 
+    {"no-convert-text-baseline-spacing", 0,
+    POPT_ARG_NONE, &sp_no_convert_text_baseline_spacing, SP_ARG_NO_CONVERT_TEXT_BASELINE_SPACING,
+    N_("Do not fix legacy (pre-0.92) files' text baseline spacing on opening."),
+    NULL},
+
     POPT_AUTOHELP POPT_TABLEEND
 };
 
