@@ -28,8 +28,7 @@
 
 #include "display/curve.h"
 #include <2geom/pathvector.h>
-#include <2geom/bezier-curve.h>
-#include <2geom/hvlinesegment.h>
+#include <2geom/curves.h>
 #include "helper/geom-curves.h"
 
 #include "svg/svg.h"
@@ -41,7 +40,7 @@
 
 #include "document.h"
 #include "desktop.h"
-#include "desktop-handles.h"
+
 #include "desktop-style.h"
 #include "ui/tools/tool-base.h"
 #include "inkscape.h"
@@ -51,16 +50,6 @@
 
 #define noPATH_VERBOSE
 
-#include "sp-factory.h"
-
-namespace {
-	SPObject* createPath() {
-		return new SPPath();
-	}
-
-	bool pathRegistered = SPFactory::instance().registerObject("svg:path", createPath);
-}
-
 gint SPPath::nodesInPath() const
 {
     return _curve ? _curve->nodes_in_path() : 0;
@@ -140,10 +129,29 @@ void SPPath::build(SPDocument *document, Inkscape::XML::Node *repr) {
 
     SPShape::build(document, repr);
 
-    // this->readAttr( "inkscape:original-d" ); // lp1299948
+    // this->readAttr( "inkscape:original-d" ); // bug #1299948
+    // Why we take the long way of doing this probably needs some explaining:
+    //
+    // Normally upon being built, reading the inkscape:original-d attribute
+    // will cause the path to actually _write to its repr_ in response to this.
+    // This is bad, bad news if the attached effect refers to a path which
+    // hasn't been constructed yet.
+    // 
+    // What will happen is the effect parameter will cause the effect to
+    // recalculate with a completely different value due to the parameter being
+    // "empty" -- even worse, an undo event might be created with the bad value,
+    // and undoing the current action could cause it to revert to the "bad"
+    // state. (After that, the referred object will be constructed and the
+    // reference will trigger the path effect to update and commit the right
+    // value to "d".)
+    //
+    // This mild nastiness here (don't recalculate effects on build) prevents a
+    // plethora of issues with effects with linked parameters doing wild and
+    // stupid things on new documents upon a mere undo.
+
     if (gchar const* s = this->getRepr()->attribute("inkscape:original-d"))
     {
-        // write it to XML, and to my curve, but don't update patheffects
+        // Write the value to _curve_before_lpe, do not recalculate effects
         Geom::PathVector pv = sp_svg_read_pathv(s);
         SPCurve *curve = new SPCurve(pv);
         
@@ -158,10 +166,17 @@ void SPPath::build(SPDocument *document, Inkscape::XML::Node *repr) {
     this->readAttr( "d" );
 
     /* d is a required attribute */
-    gchar const *d = this->getAttribute("d", NULL);
+    char const *d = this->getAttribute("d", NULL);
 
     if (d == NULL) {
-        this->setKeyValue( sp_attribute_lookup("d"), "");
+        // First see if calculating the path effect will generate "d":
+        this->update_patheffect(true);
+        d = this->getAttribute("d", NULL);
+
+        // I guess that didn't work, now we have nothing useful to write ("")
+        if (d == NULL) {
+            this->setKeyValue( sp_attribute_lookup("d"), "");
+        }
     }
 }
 
@@ -279,8 +294,9 @@ Geom::Affine SPPath::set_transform(Geom::Affine const &transform) {
 
     // Transform the original-d path if this is a valid LPE this, other else the (ordinary) path
     if (_curve_before_lpe && hasPathEffectRecursive()) {
-        if (this->hasPathEffectOfType(Inkscape::LivePathEffect::CLONE_ORIGINAL)) {
+        if (this->hasPathEffectOfType(Inkscape::LivePathEffect::CLONE_ORIGINAL) || this->hasPathEffectOfType(Inkscape::LivePathEffect::BEND_PATH)) {
             // if path has the CLONE_ORIGINAL LPE applied, don't write the transform to the pathdata, but write it 'unoptimized'
+            // also if the effect is type BEND PATH to fix bug #179842
             return transform;
         } else {
             _curve_before_lpe->transform(transform);
